/***********************************************************
 * $Id$
 *
 * FancyDB
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDBCONNECTIONPOOL_H_
#define FANCYDBCONNECTIONPOOL_H_

#define FANCYDB_CONNECTIONPOOL_BEAN "Fancy::DB::ConnectionPool"

#include <fancydb.h>
#include <fancyobject.h>

namespace Fancy {

class Map;

namespace DB {

class Connection;

class FANCYDB_API ConnectionPool: public virtual Fancy::Object {
public:
	typedef Fancy::ObjPtr<ConnectionPool> ptr_t;

    /**
     * Generate a new connection pool instance. The returned instance must
     * be activated by calling @c start() afterwards.
     * This method is equivalent to calling the options-aware method with a null options argument.
     *
     * @param minSize The minimal size of the connection pool.
     * @param maxSize The maximal size of the connection pool.
     * @param idleTimeout The time in milliseconds to wait before disposing
     *                    an idle connection.
     * @param waitTimeout The time in milliseconds to wait for a connection to become
     *                    available, when all connection are currently busy.
     *                    If no connection is available after @c waitTimeout
     *                    seconds, a time out exception is thrown.
     * @param url The database URL to connect to.
     * @param user The database user for the connections
     * @param passwd The database password used to connect to the database.
     *
     * @return A new @c ConnectionPool instance.
     */
	static ptr_t newInstance(size_t minSize, size_t maxSize,
                             unsigned idleTimeout, unsigned waitTimeout,
                             const std::string &url,
                             const std::string &user,
                             const std::string &passwd);

    /**
     * Generate a new connection pool instance. The returned instance must
     * be activated by calling @c start() afterwards.
     *
     * @param minSize The minimal size of the connection pool.
     * @param maxSize The maximal size of the connection pool.
     * @param idleTimeout The time in milliseconds to wait before disposing
     *                    an idle connection.
     * @param waitTimeout The time in milliseconds to wait for a connection to become
     *                    available, when all connection are currently busy.
     *                    If no connection is available after @c waitTimeout
     *                    seconds, a time out exception is thrown.
     * @param url The database URL to connect to.
     * @param user The database user for the connections
     * @param passwd The database password used to connect to the database.
     * @param options Database-sepcific options with strings as keys and
     *                    option-specific objects such as Boolean, Integer or
     *                    String as values. Refer to your drivers documentation for supported
     *                    options.
     *
     * @return A new @c ConnectionPool instance.
     */
	static ptr_t newInstance(size_t minSize, size_t maxSize,
                             unsigned idleTimeout, unsigned waitTimeout,
                             const std::string &url,
                             const std::string &user,
                             const std::string &passwd,
                             const Fancy::ObjPtr<Fancy::Map> &options);

	/**
	 * @return A new @c Connection instance, which is in fact shadowing a real
	 *         @c Connection instance. The original instance will be returned
	 *         to the pool as soon as the returned instance is destroyed.
	 *         It is thus advisable to let the returned connection go out of scope
	 *         as soon as the last result set has been retrieved and copied to
	 *         an application-specific dataset.
	 *         It is furthermore strongly recommended to use transactional database
	 *         access for pooled connections.
	 */
	virtual Fancy::ObjPtr<Connection> fetchConnection() = 0;

    /**
     * Starts this connection pool and starts the garbage collection.
     * Attempts to connect enough connections to satisfy the minimal pool size.
     * This triggers an exception if the database server is not up yet.
     * Therefore, in situations where the server might not be up yet,
     * set the minimal pool size to 0 and implement an initial retry loop yourself.
     */
	virtual void start() = 0;

    /**
     * Stop this connection pool and the garbage collector.
     * @c stop() is usually called inside the destructor, but
     * the user is encouraged to call stop() before the destructor.
     */
	virtual void stop() = 0;

protected:
	ConnectionPool();
	virtual ~ConnectionPool();
};

}

}

#endif /* FANCYDBCONNECTIONPOOL_H_ */
